package PlayBot::sessions::downloader::ddl;

use strict;
use warnings;
use utf8;

use JSON;
use POE;
use POE::Wheel::Run;

use PlayBot::utils::db;
use PlayBot::utils::Logging;

my $log = PlayBot::utils::Logging->new( 'STDOUT', 1 );
my $conf;

BEGIN {
    chdir "$FindBin::Bin/";
    local $/;
    open CONF, '<', 'playbot.conf';
    my $json = <CONF>;
    $conf = decode_json($json);
}

sub exec {
    my ( $kernel, $heap, $id, $url ) = @_[ KERNEL, HEAP, ARG0, ARG1 ];
    return if ( not $id or not $url );
    $log->debug("DDL '$id' '$url' ");

    # is there already a task running ?
    if ( $heap->{ddl}->{running} ) {
        push $heap->{ddl}->{tasks}, [ $id, $url ];
        return;
    }

    $heap->{ddl}->{running} = 1;

    # first we need to got filename
    my $child = POE::Wheel::Run->new(
        Program => [
            "youtube-dl",         "--restrict-filename",
            "--get-filename",     "-o",
            $conf->{save_format}, $url,
        ],
        StdoutEvent => "filename",
        StderrEvent => "devnull",
    );
    $kernel->sig_child( $child->PID, "signal_filename" );

    # we link playbot id to PID and url
    $heap->{ddl}->{pb}->{$id} = {
        child => $child->PID,
        url   => $url,
    };

    # we link wheel id to playbot id
    $heap->{ddl}->{wid}->{ $child->ID } = $id;

    # we link PID to child object
    $heap->{ddl}->{pid}->{ $child->PID } = $child;
}

sub filename {
    $log->debug("DDL");
    my ( $heap, $stdout_line, $wid ) = @_[ HEAP, ARG0, ARG1 ];
    return if ( not exists $heap->{ddl}->{wid}->{$wid} );

    my $id = $heap->{ddl}->{wid}->{$wid};
    $heap->{ddl}->{pb}->{$id}->{filename} = $stdout_line;
    $heap->{ddl}->{pb}->{$id}->{child}    = undef;
}

sub signal_filename {
    $log->debug("DDL");
    my ( $kernel, $heap, $pid, $status ) = @_[ KERNEL, HEAP, ARG1, ARG2 ];

    my $child = delete $heap->{ddl}->{pid}->{$pid};
    my $id    = delete $heap->{ddl}->{wid}->{ $child->ID };

    if ( not defined $heap->{ddl}->{pb}->{$id}->{filename} ) {
        $log->error("error getting filename for $id : $status");
        $heap->{ddl}->{pb}->{$id}->{child} = undef;

        $kernel->yield( "end_task", $id );
        return;
    }

    # we actually download the content
    $child = POE::Wheel::Run->new(
        Program => [
            "youtube-dl", "--restrict-filename",
            "-o",         "/tmp/" . $conf->{save_format},
            $heap->{ddl}->{pb}->{$id}->{url},
        ],
        StdoutEvent => "devnull",
        StderrEvent => "devnull",
    );
    $kernel->sig_child( $child->PID, "signal_ddl" );

    # we link playbot id to PID and url
    $heap->{ddl}->{pb}->{$id}->{child} = $child->PID;

    # we link wheel id to playbot id
    $heap->{ddl}->{wid}->{ $child->ID } = $id;

    # we link PID to child object
    $heap->{ddl}->{pid}->{ $child->PID } = $child;
}

sub signal_ddl {
    $log->debug("DDL");
    my ( $kernel, $heap, $pid, $status ) = @_[ KERNEL, HEAP, ARG1, ARG2 ];

    my $child = delete $heap->{ddl}->{pid}->{$pid};
    my $id    = delete $heap->{ddl}->{wid}->{ $child->ID };
    $heap->{ddl}->{pb}->{$id}->{child} = undef;

    if ( $status ne "0" ) {
        $log->error("error downloading $id : $status");
        rm( $kernel, $heap, $id,
            "/tmp/" . $heap->{ddl}->{pb}->{$id}->{filename} . ".part" );
        return;
    }

    # we save the content
    $child = POE::Wheel::Run->new(
        Program => [
            "mv",
            "/tmp/" . $heap->{ddl}->{pb}->{$id}->{filename},
            $conf->{save_folder} . "/" . $heap->{ddl}->{pb}->{$id}->{filename},
        ],
        StdoutEvent => "devnull",
        StderrEvent => "devnull",
    );
    $kernel->sig_child( $child->PID, "signal_move" );

    # we link playbot id to PID and url
    $heap->{ddl}->{pb}->{$id}->{child} = $child->PID;

    # we link wheel id to playbot id
    $heap->{ddl}->{wid}->{ $child->ID } = $id;

    # we link PID to child object
    $heap->{ddl}->{pid}->{ $child->PID } = $child;
}

sub signal_move {
    $log->debug("DDL");
    my ( $kernel, $heap, $pid, $status ) = @_[ KERNEL, HEAP, ARG1, ARG2 ];

    my $child = delete $heap->{ddl}->{pid}->{$pid};
    my $id    = delete $heap->{ddl}->{wid}->{ $child->ID };
    $heap->{ddl}->{pb}->{$id}->{child} = undef;

    if ( $status ne "0" ) {
        $log->error("error moving $id from tmp : $status");

        # rm file
        rm( $kernel, $heap, $id,
            "/tmp/" . $heap->{ddl}->{pb}->{$id}->{filename} );
        return;
    }

    # we update db
    eval {
        PlayBot::utils::db::set_filename( $id,
            $heap->{ddl}->{pb}->{$id}->{filename} );
        $kernel->yield( "end_task", $id );
    };
    if ($a) {
        $log->error($@);
        rm(
            $kernel,
            $heap,
            $id,
            $conf->{save_folder} . "/" . $heap->{ddl}->{pb}->{$id}->{filename},
        );
    }
}

sub rm {
    my ( $kernel, $heap, $id, $filename ) = @_;

    my $child = POE::Wheel::Run->new(
        Program     => [ "rm", $filename ],
        StdoutEvent => "devnull",
        StderrEvent => "devnull",
    );
    $kernel->sig_child( $child->PID, "signal_rm" );

    # we link playbot id to PID and url
    $heap->{ddl}->{pb}->{$id}->{child} = $child->PID;

    # we link wheel id to playbot id
    $heap->{ddl}->{wid}->{ $child->ID } = $id;

    # we link PID to child object
    $heap->{ddl}->{pid}->{ $child->PID } = $child;
}

sub signal_rm {
    $log->debug("DDL");
    my ( $kernel, $heap, $pid, $status ) = @_[ KERNEL, HEAP, ARG1, ARG2 ];

    my $child = delete $heap->{ddl}->{pid}->{$pid};
    my $id    = delete $heap->{ddl}->{wid}->{ $child->ID };
    $heap->{ddl}->{pb}->{$id}->{child} = undef;

    if ( $status ne "0" ) {
        $log->critical("cannot remove file");
    }

    $kernel->yield( "end_task", $id );
}

sub end_task {
    my ( $kernel, $heap, $id ) = @_[ KERNEL, HEAP, ARG0 ];

    $heap->{ddl}->{running} = 0;
    delete $heap->{ddl}->{pb}->{$id};

    if ( @{ $heap->{ddl}->{tasks} } ) {
        my $task = shift $heap->{ddl}->{tasks};
        $kernel->yield( ddl => $task->[0], $task->[1] );
    }
}

1;
